{                                                                           }
{ File:       DirIterThread.pas                                             }
{ Function:   Abstract TThread descendant used to iterate over files in a   }
{             directory (and subdirectories). Override HandleFile and       }
{             HandleDirectory for functionality.                            }
{ Language:   Delphi 8 for the Microsoft .NET framework                     }
{ Author:     Rudy Velthuis                                                 }
{ Copyright:  (c) 2003 Rudy Velthuis                                        }
{ Disclaimer: This code is freeware. All rights are reserved.               }
{             This code is provided as is, expressly without a warranty     }
{             of any kind. You use it at your own risk.                     }
{                                                                           }
{             If you use this code, please credit me.                       }
{                                                                           }

unit DirIterThread;

interface

{$WARN UNIT_PLATFORM OFF}

uses
  Classes, SysUtils;

type
  TDirectoryIteratorThread = class(TThread)
  private
    FDirectory: string;
    FSubdirectories: Boolean;
    FSearchMasks: TStringList;
    function GetMask(Index: Integer): string;
  protected
    procedure Execute; override;
  public
    constructor Create(const Directory, SearchMasks: string;
      Subdirectories: Boolean);
    destructor Destroy; override;
    procedure HandleFile(const FileName: string; Attr: Integer); virtual;
      abstract;
    procedure HandleDirectory(const Directory: string); virtual; abstract;
    procedure HandleError(const Directory: string; Error: Exception);
    function SearchDirectory(const Dir: string): Boolean;
    property Masks[Index: Integer]: string read GetMask;
  end;

implementation

uses System.IO, GreppolaUtils;

{ TDirectoryIteratorThread }

constructor TDirectoryIteratorThread.Create(const Directory,
  SearchMasks: string; Subdirectories: Boolean);
begin
  inherited Create(True);
  FDirectory := IncludeTrailingPathDelimiter(Directory);
  FSubdirectories := Subdirectories;
  FSearchMasks := TStringList.Create;
  FSearchMasks.Delimiter := ';';
  if SearchMasks = '' then
    FSearchMasks.DelimitedText := '*.*'
  else
    FSearchMasks.DelimitedText := SearchMasks;
end;

destructor TDirectoryIteratorThread.Destroy;
begin
  FSearchMasks.Free;
  inherited;
end;

procedure TDirectoryIteratorThread.Execute;
begin
  SearchDirectory(FDirectory);
end;

function TDirectoryIteratorThread.GetMask(Index: Integer): string;
begin
  Result := FSearchMasks[Index];
end;

procedure TDirectoryIteratorThread.HandleError(const Directory: string;
  Error: Exception);
begin
  // Do nothing!
end;

function TDirectoryIteratorThread.SearchDirectory(const Dir: string): Boolean;
var
  Entries, Directories: array of string;
  Mask, Entry, SubDirectory: string;
  SearchResult: Boolean;
begin
  try
    Result := False;
    for Mask in FSearchMasks do
    begin
      Entries := Directory.GetFiles(Dir, Mask);
      Result := Length(Entries) > 0;
      for Entry in Entries do
      begin
        if Terminated then Exit;
        HandleFile(Entry, 0);
      end;
      HandleDirectory(Dir);
    end;

    if FSubDirectories and not Terminated then
    begin
      Directories := Directory.GetDirectories(Dir);
      for SubDirectory in Directories do
      begin
        if Terminated then Exit;
        SearchResult := SearchDirectory(SubDirectory);
        Result := Result or SearchResult;
      end;
    end;
  except
    on E: Exception do
    begin
      HandleError(Dir, E);
      Result := False;
    end;
  end;
end;

end.

