{                                                                           }
{ File:       FEditorFrame.pas                                              }
{ Function:   Single editor frame for each page in the FEditor unit, for    }
{             the Greppola application.                                     }
{ Language:   Delphi 8 for the Microsoft .NET framework                     }
{ Author:     Rudy Velthuis                                                 }
{ Copyright:  (c) 2003, 2004 Rudy Velthuis                                  }
{ Disclaimer: This code is freeware. All rights are reserved.               }
{             This code is provided as is, expressly without a warranty     }
{             of any kind. You use it at your own risk.                     }
{                                                                           }

unit FEditorFrame;

{$WARN UNIT_PLATFORM OFF}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, System.ComponentModel, GreppolaUtils;

type
  TEditorFrameNotifyEvent = procedure(Sender: TObject; Line, Column: Integer;
    Inserting, HasSelection, CanUndo: Boolean);

  TEditorFrame = class(TFrame)
    RichEdit: TRichEdit;
    procedure RichEditSelectionChange(Sender: TObject);
    procedure RichEditKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
  private
    FRegex: STRRegex;
    FFileName: string;
    FOnChange: TEditorFrameNotifyEvent;
    FInserting: Boolean;
    procedure UpdateStatus;
    function GetHasSelection: Boolean;
  public
    function GetCanUndo: Boolean;
  public
    procedure GetCaret(var Line, Column: Integer);
    procedure OpenFile(AFileName: string);
    procedure SelectLine(Line, Column, Length: Integer);
    procedure Search(Regex: STRRegex; WholeWord: Boolean);
    property Regex: STRRegex read FRegex write FRegex;
    property FileName: string read FFileName write FFileName;
    property HasSelection: Boolean read GetHasSelection;
    property CanUndo: Boolean read GetCanUndo;
    property OnChange: TEditorFrameNotifyEvent read FOnChange write FOnChange;
  end;

implementation

{$R *.nfm}

uses GreppolaConsts;

{ TEditorFrame }

procedure TEditorFrame.OpenFile(AFileName: string);
begin
  FFileName := AFileName;
  RichEdit.Lines.LoadFromFile(FileName);
  FInserting := True;
end;

procedure TEditorFrame.SelectLine(Line, Column, Length: Integer);
var
  Index: Integer;
begin
  if Line = -1 then
    Exit;
  Index := RichEdit.Perform(EM_LINEINDEX, Line, 0);
  RichEdit.SelStart := Index + Column;
  RichEdit.SelLength := Length;
  RichEdit.Perform(EM_SCROLLCARET, 0, 0);
  Windows.SetFocus(RichEdit.Handle);
end;

procedure TEditorFrame.UpdateStatus;
var
  Line, Column: Integer;
begin
  if Assigned(FOnChange) then
  begin
    GetCaret(Line, Column);
    FOnChange(RichEdit, Line, Column, FInserting, HasSelection, CanUndo);
  end;
end;

procedure TEditorFrame.RichEditSelectionChange(Sender: TObject);
begin
  UpdateStatus;
end;

function TEditorFrame.GetHasSelection: Boolean;
begin
  Result := RichEdit.SelLength > 0;
end;

procedure TEditorFrame.RichEditKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (Shift = []) and (Key = VK_INSERT)then
    FInserting := not FInserting;
  UpdateStatus;
end;

procedure TEditorFrame.GetCaret(var Line, Column: Integer);
const
  EM_EXLINEFROMCHAR = WM_USER + 54;
begin
  Line := RichEdit.Perform(EM_EXLINEFROMCHAR, 0, RichEdit.SelStart) + 1;
  Column := RichEdit.SelStart - RichEdit.Perform(EM_LINEINDEX, Line - 1, 0) + 1;
end;

// Is it possible to use RichEdit.Text and entire buffer, to cut things short?
procedure TEditorFrame.Search(Regex: STRRegex; WholeWord: Boolean);
var
  Match: STRMatch;
  I: Integer;
  Line, Column: Integer;
begin
  if RichEdit.SelLength > 0 then
    RichEdit.SelStart := RichEdit.SelStart + 1;
  GetCaret(Line, Column);
  for I := Line - 1 to RichEdit.Lines.Count - 1 do
  begin
    Match := Regex.Match(RichEdit.Lines[I], Column - 1);
    Column := 1;
    if Match.Success and
       WordCheck(RichEdit.Lines[I],
         TMatchData.Create(I, Match.Index, Match.Length, Regex)) then
    begin
      SelectLine(I, Match.Index, Match.Length);
      Exit;
    end;
  end;
  ShowMessageFmt(SNoOccurrenceFound, [Regex.ToString]);
end;

function TEditorFrame.GetCanUndo: Boolean;
begin
  Result := RichEdit.CanUndo;
end;

end.
