unit GreppolaOptions;

interface

{$WARN UNIT_PLATFORM OFF}

uses
  Graphics, IniFiles, GreppolaConsts;

type
  TAppearanceOptions = record
    FileFont: TFont;
    TextFont: TFont;
    BackColor: TColor;
    UnderlineColor: TColor;
    HighlightStyle: Boolean;
    procedure Initialize;
    procedure Finish;
    procedure Setup(AFileFont, ATextFont: TFont;
      ABackColor, AUnderlineColor: TColor);
    procedure Load(Ini: TIniFile);
    procedure Save(Ini: TIniFile);
  end;

  TToolsOptions = record
    DefaultEditor: string;
    procedure Initialize;
    procedure Finish;
    procedure Setup(AEditor: string);
    procedure Load(Ini: TIniFile);
    procedure Save(Ini: TIniFile);
  end;

  TSearchChangedOption = (scoUnknown, scoLastWeek, scoLastMonth, scoLastYear, scoUserDate);
  TSearchDateOption = (sdoModification, sdoCreation, sdoAccess);
  TSearchSizeOption = (ssoUnknown, ssoSmall, ssoMiddle, ssoLarge, ssoSize);
  TSearchSizeSizeOption = (sssoAtLeast, sssoAtMost);

  TSearchOptions = record
    ChangedOption: TSearchChangedOption;
    DateOption: TSearchDateOption;
    FromDate: TDateTime;
    ToDate: TDateTime;
    SizeOption: TSearchSizeOption;
    SizeSizeOption: TSearchSizeSizeOption;
    Size: Integer;
    procedure Initialize;
    procedure Finish;
    procedure Setup;
    procedure Load(Ini: TIniFile);
    procedure Save(Ini: TIniFile);
  end;

  TGreppolaOptions = record
    Search: TSearchOptions;
    Tools: TToolsOptions;
    Appearance: TAppearanceOptions;
    procedure Initialize;
    procedure Finish;
    procedure Load(Ini: TIniFile);
    procedure Save(Ini: TIniFile);
  end;

var
  GlobalOptions: TGreppolaOptions;

implementation

{ TToolsOptions }

procedure TToolsOptions.Finish;
begin
  // nothing
end;

procedure TToolsOptions.Initialize;
begin
  // nothing;
end;

procedure TToolsOptions.Load(Ini: TIniFile);
begin
  // nothing;
end;

procedure TToolsOptions.Save(Ini: TIniFile);
begin
  // nothing;
end;

procedure TToolsOptions.Setup(AEditor: string);
begin
  DefaultEditor := AEditor;
end;

{ TAppearanceOptions }

procedure TAppearanceOptions.Finish;
begin
  TextFont.Free;
  FileFont.Free;
end;

procedure TAppearanceOptions.Initialize;
begin
  if not Assigned(FileFont) then
    FileFont := TFont.Create;
  if not Assigned(TextFont) then
    TextFont := TFont.Create;
end;

procedure ReadFont(Ini: TIniFile; const Key: string; Font: TFont);
begin
  Font.Name := Ini.ReadString(CFonts, Key + ' Name', Font.Name);
  Font.Size := Ini.ReadInteger(CFonts, Key + ' Size', Font.Size);
  Font.Color := Ini.ReadInteger(CFonts, Key + ' Color', Integer(Font.Color));
end;

procedure TAppearanceOptions.Load(Ini: TIniFile);
begin
  ReadFont(Ini, CFileFont, FileFont);
  ReadFont(Ini, CTextFont, TextFont);
  BackColor := Ini.ReadInteger(COptions, CBackgroundColor, clWindow);
  UnderlineColor := Ini.ReadInteger(COptions, CFoundColor, clRed);
  HighlightStyle := Ini.ReadBool(COptions, CHighlightStyle, False);
end;

procedure WriteFont(Ini: TIniFile; const Key: string; Font: TFont);
begin
  Ini.WriteString(CFonts, Key + ' Name', Font.Name);
  Ini.WriteInteger(CFonts, Key + ' Size', Font.Size);
  Ini.WriteInteger(CFonts, Key + ' Color', Integer(Font.Color));
end;

procedure TAppearanceOptions.Save(Ini: TIniFile);
begin
  WriteFont(Ini, CFileFont, FileFont);
  WriteFont(Ini, CTextFont, TextFont);
  Ini.WriteInteger(COptions, CBackgroundColor, BackColor);
  Ini.WriteInteger(COptions, CFoundColor, UnderlineColor);
  Ini.WriteBool(COptions, CHighlightStyle, HighlightStyle);
end;

procedure TAppearanceOptions.Setup(AFileFont, ATextFont: TFont;
  ABackColor, AUnderlineColor: TColor);
begin
  FileFont.Assign(AFileFont);
  TextFont.Assign(ATextFont);
  BackColor := ABackColor;
  UnderlineColor := AUnderlineColor;
end;

{ TGreppolaOptions }

procedure TGreppolaOptions.Finish;
begin
  Search.Finish;
  Appearance.Finish;
  Tools.Finish;
end;

procedure TGreppolaOptions.Initialize;
begin
  Search.Initialize;
  Tools.Initialize;
  Appearance.Initialize;
end;

procedure TGreppolaOptions.Load(Ini: TIniFile);
begin
  Search.Load(Ini);
  Tools.Load(Ini);
  Appearance.Load(Ini);
end;

procedure TGreppolaOptions.Save(Ini: TIniFile);
begin
  Search.Save(Ini);
  Tools.Save(Ini);
  Appearance.Save(Ini);
end;

{ TSearchOptions }

procedure TSearchOptions.Finish;
begin
end;

procedure TSearchOptions.Initialize;
begin
  FromDate := TDateTime.MinValue;
  ToDate := TDateTime.MinValue;
end;

procedure TSearchOptions.Load(Ini: TIniFile);
begin
  ChangedOption := TSearchChangedOption(Ini.ReadInteger(StrSearchOptions, StrChangedOption, 0));
  DateOption := TSearchDateOption(Ini.ReadInteger(StrSearchOptions, StrDateOption, 0));
  FromDate := Ini.ReadDate(StrSearchOptions, StrFromDate, 0.0);
  ToDate := Ini.ReadDate(StrSearchOptions, StrToDate, 0.0);
  SizeOption := TSearchSizeOption(Ini.ReadInteger(StrSearchOptions, StrSizeOption, 0));
  SizeSizeOption := TSearchSizeSizeOption(Ini.ReadInteger(StrSearchOptions, StrSizeSizeOption, 0));
  Size := Ini.ReadInteger(StrSearchOptions, StrSize, 0);
end;

procedure TSearchOptions.Save(Ini: TIniFile);
begin
  Ini.WriteInteger(StrSearchOptions, StrChangedOption, Integer(ChangedOption));
  Ini.WriteInteger(StrSearchOptions, StrDateOption, Integer(DateOption));
  Ini.WriteDate(StrSearchOptions, StrFromDate, FromDate);
  Ini.WriteDate(StrSearchOptions, StrToDate, ToDate);
  Ini.WriteInteger(StrSearchOptions, StrSizeOption, Integer(SizeOption));
  Ini.WriteInteger(StrSearchOptions, StrSizeSizeOption, Integer(SizeSizeOption));
  Ini.WriteInteger(StrSearchOptions, StrSize, Size);
end;

procedure TSearchOptions.Setup;
begin
end;

initialization
  GlobalOptions.Initialize;

finalization
  GlobalOptions.Finish;

end.
