{                                                                           }
{ File:       Greppola.SearchThread.pas                                     }
{ Function:   Specialized TDirectoryIteratorThread for the Greppola         }
{             Application.                                                  }
{ Language:   Delphi 8 for the Microsoft .NET framework                     }
{ Author:     Rudy Velthuis                                                 }
{ Copyright:  (c) 2003 Rudy Velthuis                                        }
{ Disclaimer: This code is freeware. All rights are reserved.               }
{             This code is provided as is, expressly without a warranty     }
{             of any kind. You use it at your own risk.                     }
{                                                                           }
{             If you use this code, please credit me.                       }
{                                                                           }

{$IFNDEF CLR}
{$MESSAGE ERROR 'Only in .NET'}
{$ENDIF}

unit SearchThread;

interface

{$WARN UNIT_PLATFORM OFF}

uses
  DirIterThread, Classes,
  System.Text.RegularExpressions, GreppolaUtils, System.IO;

type
  TAddDirectoryEvent = procedure(Directory: string; Info: TStrings) of object;
  TGetTime = function(Info: FileInfo): TDateTime of object;

  TSearchThread = class(TDirectoryIteratorThread)
  private
    FMinSize: Int64;
    FMaxSize: Int64;
    FMinDate: TDateTime;
    FMaxDate: TDateTime;
    FGetTime: TGetTime;
    FCurrentDirectory: string;
    FCaseChecks: Boolean;
    FWholeWord: Boolean;
    FTextFiles: Boolean;
    FSearchedCount: Integer;
    FFoundCount: Integer;
    FLineCount: Integer;
    FError: Integer;
    FErrorString: string;
    FPassInfoList: TStringList;
    FPattern: string;
    FRegex: STRRegex;
    FOnAddDirectory: TAddDirectoryEvent;
    procedure LoadFile(FileName: string; TextLines: TStrings);
    procedure AddMatch(const FileName: string; TextLines: TStringList;
      var First: Boolean; LineIndex: Integer; RegMatch: STRMatch;
      WholeWord: Boolean);
    function IsValidDate(Info: FileInfo): Boolean;
    function IsValidSize(Info: FileInfo): Boolean;
    procedure SetSizeLimits;
    function GetAccessTime(Info: FileInfo): TDateTime;
    function GetCreationTime(Info: FileInfo): TDateTime;
    function GetModificationTime(Info: FileInfo): TDateTime;
    procedure SetDateLimits;
  protected
    procedure GrepSearch(const FileName: string; Lines: TStringList;
      CaseChecks, WholeWord: Boolean);
    procedure DoAddDirectory;
    procedure ShowError;
  public
    procedure HandleFile(const FileName: string; Attr: Integer); override;
    procedure HandleDirectory(const Directory: string); override;
    constructor Create(const Directory, SearchMasks: string;
      Subdirectories: Boolean; Pattern: string;
      CaseChecks, WholeWord, TextFiles: Boolean);
    destructor Destroy; override;
    property OnAddDirectory: TAddDirectoryEvent read FOnAddDirectory
      write FOnAddDirectory;
    property LineCount: Integer read FLineCount;
    property FoundCount: Integer read FFoundCount;
    property SearchedCount: Integer read FSearchedCount;
  end;

implementation

uses
  SysUtils, DateUtils, Dialogs, GreppolaConsts, GreppolaOptions;

{ TSearchThread }

procedure TSearchThread.DoAddDirectory;
begin
  if Assigned(FOnAddDirectory) then
    FOnAddDirectory(FCurrentDirectory, FPassInfoList);
end;

constructor TSearchThread.Create(const Directory, SearchMasks: string;
  Subdirectories: Boolean; Pattern: string;
  CaseChecks, WholeWord, TextFiles: Boolean);
var
  Options: STRRegexOptions;
begin
  inherited Create(Directory, SearchMasks, Subdirectories);
  FCaseChecks := CaseChecks;
  FWholeWord := WholeWord;
  FTextFiles := TextFiles;
  FPassInfoList := TStringList.Create;
  FPattern := Pattern;
  if CaseChecks then
    Options := STRRegexOptions.None
  else
    Options := STRRegexOptions.IgnoreCase;
  SetDateLimits;
  SetSizeLimits;
  FRegex := STRRegex.Create(Pattern, Options);
end;

destructor TSearchThread.Destroy;
begin
  FPassInfoList.Free;
  inherited;
end;

procedure TSearchThread.SetDateLimits;
begin
  FMaxDate := IncDay(Date, 1);
  case GlobalOptions.Search.ChangedOption of
    scoLastWeek:
      FMinDate := IncDay(Date, -7);
    scoLastMonth:
      FMinDate := IncMonth(Date, -1);
    scoLastYear:
      FMinDate := IncYear(Date, -1);
    scoUserDate:
      begin
        FMinDate := GlobalOptions.Search.FromDate;
        FMaxDate := GlobalOptions.Search.ToDate;
      end;
    else
      FMinDate := 0;
  end;
  case GlobalOptions.Search.DateOption of
    sdoAccess:
      FGetTime := GetAccessTime;
    sdoCreation:
      FGetTime := GetCreationTime;
    else
      FGetTime := GetModificationTime;
  end;
end;

var
  TheSizes: array[TSearchSizeOption] of
    record Min: Int64; Max: Int64; end = (
      (Min: 0;               Max: $7FFFFFFFFFFFFFFF), // Unknown
      (Min: 0;               Max: 100 * 1024),        // Small
      (Min: 100 * 1024 + 1;  Max: 1024 * 1024),       // Middle
      (Min: 1024 * 1024 + 1; Max: $7FFFFFFFFFFFFFFF), // Large
      (Min: 0;               Max: $7FFFFFFFFFFFFFFF)  // User size
    );

procedure TSearchThread.SetSizeLimits;
var
  Option: TSearchSizeOption;
begin
  Option := GlobalOptions.Search.SizeOption;
  if Option = ssoSize then
  begin
    if GlobalOptions.Search.SizeSizeOption = sssoAtLeast then
    begin
      FMinSize := GlobalOptions.Search.Size * 1024;
      FMaxSize := $7FFFFFFFFFFFFFFF;
    end
    else
    begin
      FMinSize := 0;
      FMaxSize := GlobalOptions.Search.Size * 1024;
    end;
  end
  else
  begin
    FMinSize := TheSizes[Option].Min;
    FMaxSize := TheSizes[Option].Max;
  end;
end;

procedure TSearchThread.AddMatch(const FileName: string;
  TextLines: TStringList; var First: Boolean; LineIndex: Integer;
  RegMatch: STRMatch; WholeWord: Boolean);
var
  MatchData: TMatchData;
begin
  begin
    MatchData := TMatchData.Create(LineIndex, RegMatch.Index, RegMatch.Length, FRegex);
    if not WholeWord or WordCheck(TextLines[LineIndex], MatchData) then
    begin
      if First then
      begin
        Inc(FFoundCount);
        FPassInfoList.Add(FileName);
        First := False;
      end;
      FPassInfoList.AddObject(TextLines[LineIndex], MatchData);
      Inc(FLineCount);
    end
    else
      MatchData.Free;
  end;
end;

procedure TSearchThread.LoadFile(FileName: string; TextLines: TStrings);
var
  Stream: TStream;
begin
  // Can't use LoadFromFile, since that uses fmShareDenyWrite
  Stream := TFileStream.Create(FileName, fmOpenRead or fmShareDenyNone);
  try
    TextLines.LoadFromStream(Stream);
  finally
    Stream.Free;
  end;
end;

procedure TSearchThread.GrepSearch(const FileName: string;
  Lines: TStringList; CaseChecks, WholeWord: Boolean);
var
  LineIndex: Integer;
  TextLines: TStringList;
  RegMatch: STRMatch;
  First: Boolean;
begin
  try
    TextLines := TStringList.Create;
    try
      LoadFile(FileName, TextLines);

      First := True;
      for LineIndex := 0 to TextLines.Count - 1 do
      begin
        if Terminated then
          Exit;
        RegMatch := FRegex.Match(TextLines[LineIndex]);
        if RegMatch.Success then
          AddMatch(FileName, TextLines, First, LineIndex, RegMatch, WholeWord);
      end;
    finally
      TextLines.Free;
    end;
  except
    FPassInfoList.Add(Format(SCouldntLoad, [FileName]));
  end;
end;

procedure TSearchThread.HandleDirectory(const Directory: string);
begin
  FCurrentDirectory := Directory;
  Synchronize(DoAddDirectory);
  FPassInfoList.Clear;
end;

function TSearchThread.GetCreationTime(Info: FileInfo): TDateTime;
begin
  Result := Info.CreationTime;
end;

function TSearchThread.GetModificationTime(Info: FileInfo): TDateTime;
begin
  Result := Info.LastWriteTime;
end;

function TSearchThread.GetAccessTime(Info: FileInfo): TDateTime;
begin
  Result := Info.LastAccessTime;
end;

function TSearchThread.IsValidDate(Info: FileInfo): Boolean;
var
  Date: TDateTime;
begin
  Date := FGetTime(Info);
  Result := (FMinDate <= Date) and (Date <= FMaxDate);
end;

function TSearchThread.IsValidSize(Info: FileInfo): Boolean;
var
  Size: Int64;
begin
  Size := Info.Length;
  Result := (FMinSize <= Size) and (Size <= FMaxSize);
end;

procedure TSearchThread.HandleFile(const FileName: string;
  Attr: Integer);
var
  Info: FileInfo;
begin
  inherited;

  Info := FileInfo.Create(FileName);

  // check for textfiles option.
  if (not FTextFiles or FileIsText(FileName)) and
     IsValidDate(Info) and
     IsValidSize(Info) then
  begin
    Inc(FSearchedCount);
    GrepSearch(FileName, FPassInfoList, FCaseChecks, FWholeWord);
  end;
end;

procedure TSearchThread.ShowError;
begin
  ShowMessageFmt(SErrorMessage, [FError, FErrorString]);
end;

end.
